from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        # Get the estate from the UPRN field
        estate = kwargs.get("uprn")

        # Parse the council's website
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        # Get a list of lists of estates and their collection days, then check for a match on estate name
        collection_days = [
            item.text.strip().replace("\xa0", " ").split(" - ")
            for item in soup.find(
                "div",
                {
                    "class": "field field--name-localgov-paragraphs field--type-entity-reference-revisions field--label-hidden field__items"
                },
            ).find_all("li")
        ]
        result = [
            result for result in collection_days if result[0].lower() == estate.lower()
        ]

        # If there is a match, we can process it by getting the next eight dates for that day. Else, raise an exception.
        if result is not None:
            day_number = days_of_week.get(result[0][1].split()[0])
            collection_dates = get_weekday_dates_in_period(
                datetime.now(), day_number, 8
            )

            for date in collection_dates:
                dict_data = {
                    "type": f"Weekly collection",
                    "collectionDate": date,
                }
                data["bins"].append(dict_data)
        else:
            raise ValueError("Estate not found on website.")

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        requests.packages.urllib3.disable_warnings()
        response = requests.get(
            f"https://maldon.suez.co.uk/maldon/ServiceSummary?uprn={uprn}",
            headers={"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"},
        )
        if response.status_code != 200:
            raise ValueError("No bin data found for provided UPRN.")

        soup = BeautifulSoup(response.text, features="html.parser")
        collections = soup.find_all("div", {"class": "panel"})
        for c in collections:
            binType = c.find("div", {"class": "panel-heading"}).get_text(strip=True)
            collectionDate = ""
            rows = c.find("div", {"class": "panel-body"}).find_all(
                "div", {"class": "row"}
            )
            for row in rows:
                if row.find("strong").get_text(strip=True).lower() == "next collection":
                    collectionDate = row.find(
                        "div", {"class": "col-sm-9"}
                    ).get_text(strip=True)

            if collectionDate != "":
                collection_data = {
                    "type": binType,
                    "collectionDate": collectionDate,
                }
                data["bins"].append(collection_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get postcode and UPRN from kwargs
        # user_postcode = kwargs.get("postcode")
        user_uprn = kwargs.get("uprn")
        # check_postcode(user_postcode)
        check_uprn(user_uprn)
        url = "https://www.stratford.gov.uk/waste-recycling/when-we-collect.cfm/part/calendar"
        payload = {
            "frmAddress1": "",
            "frmAddress2": "",
            "frmAddress3": "",
            "frmAddress4": "",
            "frmPostcode": "",
            "frmUPRN": user_uprn,
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}

        requests.packages.urllib3.disable_warnings()
        response = requests.request("POST", url, data=payload, headers=headers)

        # Make a BS4 object
        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()


                # Find the table
        table = soup.find('table', class_='table')

        data = {"bins": []}

        if table:
            # Extract the column headers (bin names)
            column_headers = [header.text.strip() for header in table.select('thead th.text-center strong')]
            
            # Extract the rows containing collection information
            collection_rows = table.select('tbody tr')

            # Create a dictionary to store the next date for each bin
            next_collection_dates = {bin: None for bin in column_headers}


            # Iterate through the rows
            for row in collection_rows:
                # Get the date from the first cell
                date_str = row.find('td').text.strip()
                date_obj = datetime.strptime(date_str, '%A, %d/%m/%Y')
                
                # Get the collection information for each bin (td elements with title attribute)
                collection_info = [cell['title'] if cell['title'] else 'Not Collected' for cell in row.select('td.text-center')]

                # Iterate through each bin type and its collection date
                for bin, status in zip(column_headers, collection_info):
                    # If the bin hasn't had a collection date yet or the new date is earlier, update it
                    if status != "Not Collected" and (not next_collection_dates[bin] or date_obj < next_collection_dates[bin]):
                        next_collection_dates[bin] = date_obj

            data["bins"]  = [{"type": bin, "collectionDate": next_date.strftime(date_format)} for bin, next_date in next_collection_dates.items()]
        else:
            print("Table not found in the HTML content.")

        return data



from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        headers = {
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Accept-Language": "en-GB,en-US;q=0.9,en;q=0.8",
            "Connection": "keep-alive",
            "Host": "online.torbay.gov.uk",
            "Origin": "https://www.torbay.gov.uk",
            "Referer": "https://www.torbay.gov.uk/",
            "sec-ch-ua": '"Chromium";v="110", "Not A(Brand";v="24", "Google Chrome";v="110"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-site",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36",
        }
        requests.packages.urllib3.disable_warnings()
        response = requests.get(
            f"https://online.torbay.gov.uk/services.bartec/collections?uprn={uprn}",
            headers=headers,
        )
        if response.status_code != 200:
            raise ValueError("No bin data found for provided UPRN.")
        json_data = json.loads(response.text)

        data = {"bins": []}
        for c in json_data:
            dict_data = {
                "type": c["Service"].replace("Empty ", "").strip(),
                "collectionDate": datetime.strptime(
                    c["NextCollection"].strip(), "%d %B %Y"
                ).strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from datetime import datetime

from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://selfservice.preston.gov.uk/service/Forms/FindMyNearest.aspx?Service=bins"

        data = {"bins": []}

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_paon(user_paon)
        check_postcode(user_postcode)

        # Set up Selenium to run 'headless'
        options = webdriver.ChromeOptions()
        options.add_argument("--headless")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-gpu")
        options.add_argument("--disable-dev-shm-usage")
        options.add_experimental_option("excludeSwitches", ["enable-logging"])

        # Create Selenium webdriver
        driver = webdriver.Chrome(options=options)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property
        inputElement_address = driver.find_element(
            By.ID,
            "MainContent_txtAddress",
        )

        inputElement_address.send_keys(user_paon)
        inputElement_address.send_keys(" ")
        inputElement_address.send_keys(user_postcode)

        driver.find_element(
            By.ID,
            "btnSearch",
        ).click()

        # Wait for the 'Select your property' dropdown to appear and select the first result
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "MainContent_ddlSearchResults"))
        )
        # Create a 'Select' for it, then select the first address in the list
        # (Index 0 is "Make a selection from the list")
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_index(1)

        # Wait for the 'View more' link to appear, then click it to get the full set of dates
        viewMoreLink = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.LINK_TEXT, "View more collection dates"))
        )
        viewMoreLink.click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        topLevelSpan = soup.find(
            "span",
            id="lblCollectionDates"
        )

        collectionDivs = topLevelSpan.findChildren(recursive=False)
        for collectionDiv in collectionDivs:
            # Each date has two child divs - the date and the bin type
            # However both strings are in <b> tags so there are 4 children, the text is at:
            # Index 1 - date string i.e. 'Monday 01/01/2023'
            # Index 3 - bin type i.e. 'General waste'
            typeAndDateDivs = collectionDiv.findChildren()

            # Strip the day (i.e. Monday) out of the collection date string for parsing
            dateString = typeAndDateDivs[1].text.split(' ')[1]

            data["bins"].append(
                {
                    "type": typeAndDateDivs[3].text,
                    "collectionDate": datetime.strptime(dateString, "%d/%m/%Y").strftime(date_format)
                }
            )

        return data



from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://forms.north-norfolk.gov.uk/outreach/BinCollectionDays.ofml"

        data = {"bins": []}

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_paon(user_paon)
        check_postcode(user_postcode)

        # Set up Selenium to run 'headless'
        options = webdriver.ChromeOptions()
        options.add_argument("--headless")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-gpu")
        options.add_argument("--disable-dev-shm-usage")
        options.add_experimental_option("excludeSwitches", ["enable-logging"])

        # Create Selenium webdriver
        driver = webdriver.Chrome(options=options)
        driver.get(page)

        # Populate postcode field
        inputElement_postcode = driver.find_element(
            By.ID,
            "F_Address_subform:Postcode",
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        driver.find_element(
            By.ID,
            "BA_Address_subform:Search_button",
        ).click()

        # Wait for the 'Select address' dropdown to appear
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.XPATH, "//select[@id='F_Address_subform:Id']"))
        )
        # Create a 'Select' for it, then select the matching house number/name option
        dropdownSelect = Select(dropdown)
        matchingOptions = [o for o in dropdownSelect.options if user_paon.lower() in o.text.lower()]
        if matchingOptions:
            matchingOptions[0].click()

            # Wait for the results to appear
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.XPATH, "//div[contains(@class, 'fieldmergedcolumn')]/ul"))
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            bins_text = soup.find("div", id="Search_result_details_cps_hd")

            if bins_text:
                results = re.findall("Your next (.*?) Bin collection is ([A-Za-z]+ \\d\\d? [A-Za-z]+)",
                                     bins_text.get_text())
                if results:
                    for result in results:
                        collection_date = datetime.strptime(result[1] + " " + datetime.now().strftime("%Y"),
                                                            "%A %d %B %Y")
                        dict_data = {
                            "type": result[0],
                            "collectionDate": collection_date.strftime(date_format),
                        }
                        data["bins"].append(dict_data)

                        data["bins"].sort(
                            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
                        )
        else:
            raise ValueError("No matching address for house number/name found.")

        return data



import time

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


def get_headers(base_url: str, method: str) -> dict[str, str]:
    """
    Gets request headers
        :rtype: dict[str, str]
        :param base_url: Base URL to use
        :param method: Method to use
        :return: Request headers
    """
    headers = {
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "en-GB,en-US;q=0.9,en;q=0.8",
        "Cache-Control": "max-age=0",
        "Connection": "keep-alive",
        "Host": "service.croydon.gov.uk",
        "Origin": base_url,
        "sec-ch-ua": '"Not_A Brand";v="99", "Google Chrome";v="109", "Chromium";v="109"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": "Windows",
        "Sec-Fetch-Dest": "document",
        "Sec-Fetch-User": "?1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko)"
        " Chrome/109.0.0.0 Safari/537.36",
    }
    if method.lower() == "post":
        headers["Accept"] = "application/json, text/javascript, */*; q=0.01"
        headers["Content-Type"] = "application/x-www-form-urlencoded; charset=UTF-8"
        headers["Sec-Fetch-Mode"] = "cors"
        headers["Sec-Fetch-Mode"] = "same-origin"
        headers["X-Requested-With"] = "XMLHttpRequest"
    else:
        headers["Accept"] = (
            "text/html,application/xhtml+xml,application/xml;"
            "q=0.9,image/avif,image/webp,image/apng,*/*;"
            "q=0.8,application/signed-exchange;v=b3;q=0.9"
        )
        headers["Sec-Fetch-Mode"] = "navigate"
        headers["Sec-Fetch-Mode"] = "none"
    return headers


def get_session_storage_global() -> object:
    """
    Gets session storage global object
        :rtype: object
        :return: Session storage global object
    """
    return {
        "destination_stack": [
            "w/webpage/bin-day-enter-address",
            "w/webpage/your-bin-collection-details?context_record_id=86086077"
            "&webpage_token=5c047b2c10b4aad66bef2054aac6bea52ad7a5e185ffdf7090b01f8ddc96728f",
            "w/webpage/bin-day-enter-address",
            "w/webpage/your-bin-collection-details?context_record_id=86085229"
            "&webpage_token=cf1b8fd6213f4823277d98c1dd8a992e6ebef1fabc7d892714e5d9dade448c37",
            "w/webpage/bin-day-enter-address",
            "w/webpage/your-bin-collection-details?context_record_id=86084221"
            "&webpage_token=7f52fb51019bf0e6bfe9647b1b31000124bd92a9d95781f1557f58b3ed40da52",
            "w/webpage/bin-day-enter-address",
            "w/webpage/your-bin-collection-details?context_record_id=86083209"
            "&webpage_token=de50c265da927336f526d9d9a44947595c3aa38965aa8c495ac2fb73d272ece8",
            "w/webpage/bin-day-enter-address",
        ],
        "last_context_record_id": "86086077",
    }


def get_csrf_token(s: requests.session, base_url: str) -> str:
    """
    Gets a CSRF token
        :rtype: str
        :param s: requests.session() to use
        :param base_url: Base URL to use
        :return: CSRF token
    """
    csrf_token = ""
    response = s.get(
        base_url + "/wasteservices/w/webpage/bin-day-enter-address",
        headers=get_headers(base_url, "GET"),
    )
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()
        app_body = soup.find("div", {"class": "app-body"})
        script = app_body.find("script", {"type": "text/javascript"}).string
        p = re.compile("var CSRF = ('|\")(.*?)('|\");")
        m = p.search(script)
        csrf_token = m.groups()[1]
    else:
        raise ValueError(
            "Code 1: Failed to get a CSRF token. Please ensure the council website is online first,"
            " then open an issue on GitHub."
        )
    return csrf_token


def get_address_id(
    s: requests.session, base_url: str, csrf_token: str, postcode: str, paon: str
) -> str:
    """
    Gets the address ID
        :rtype: str
        :param s: requests.session() to use
        :param base_url: Base URL to use
        :param csrf_token: CSRF token to use
        :param postcode: Postcode to use
        :param paon: House number/address to find
        :return: address ID
    """
    address_id = "0"
    # Get the addresses for the postcode
    form_data = {
        "code_action": "search",
        "code_params": '{"search_item":"' + postcode + '","is_ss":true}',
        "fragment_action": "handle_event",
        "fragment_id": "PCF0020408EECEC1",
        "fragment_collection_class": "formtable",
        "fragment_collection_editable_values": '{"PCF0021449EECEC1":"1"}',
        "_session_storage": json.dumps(
            {
                "/wasteservices/w/webpage/bin-day-enter-address": {},
                "_global": get_session_storage_global(),
            }
        ),
        "action_cell_id": "PCL0005629EECEC1",
        "action_page_id": "PAG0000898EECEC1",
        "form_check_ajax": csrf_token,
    }
    response = s.post(
        base_url
        + "/wasteservices/w/webpage/bin-day-enter-address?webpage_subpage_id=PAG0000898EECEC1"
        "&webpage_token=faab02e1f62a58f7bad4c2ae5b8622e19846b97dde2a76f546c4bb1230cee044"
        "&widget_action=fragment_action",
        headers=get_headers(base_url, "POST"),
        data=form_data,
    )
    if response.status_code == 200:
        json_response = json.loads(response.text)
        addresses = json_response["response"]["items"]
        # Find the matching address id for the paon
        for address in addresses:
            # Check for full matches first
            if address.get("dropdown_display_field") == paon:
                address_id = address.get("id")
                break
        # Check for matching start if no full match found
        if address_id == "0":
            for address in addresses:
                if address.get("dropdown_display_field").split()[0] == paon.strip():
                    address_id = address.get("id")
                    break
        # Check match was found
        if address_id == "0":
            raise ValueError(
                "Code 2: No matching address for house number/full address found."
            )
    else:
        raise ValueError("Code 3: No addresses found for provided postcode.")
    return address_id


def get_collection_data(
    s: requests.session, base_url: str, csrf_token: str, address_id: str
) -> str:
    """
    Gets the collection data
        :rtype: str
        :param s: requests.session() to use
        :param base_url: Base URL to use
        :param csrf_token: CSRF token to use
        :param address_id: Address id to use
        :param retries: Retries count
        :return: Collection data
    """
    collection_data = ""
    if address_id != "0":
        form_data = {
            "form_check": csrf_token,
            "submitted_page_id": "PAG0000898EECEC1",
            "submitted_widget_group_id": "PWG0002644EECEC1",
            "submitted_widget_group_type": "modify",
            "submission_token": "63e9126bacd815.12997577",
            "payload[PAG0000898EECEC1][PWG0002644EECEC1][PCL0005629EECEC1][formtable]"
            "[C_63e9126bacfb3][PCF0020408EECEC1]": address_id,
            "payload[PAG0000898EECEC1][PWG0002644EECEC1][PCL0005629EECEC1][formtable]"
            "[C_63e9126bacfb3][PCF0021449EECEC1]": "1",
            "payload[PAG0000898EECEC1][PWG0002644EECEC1][PCL0005629EECEC1][formtable]"
            "[C_63e9126bacfb3][PCF0020072EECEC1]": "Next",
            "submit_fragment_id": "PCF0020072EECEC1",
            "_session_storage": json.dumps({"_global": get_session_storage_global()}),
            "_update_page_content_request": 1,
            "form_check_ajax": csrf_token,
        }
        response = s.post(
            base_url
            + "/wasteservices/w/webpage/bin-day-enter-address?webpage_subpage_id=PAG0000898EECEC1"
            "&webpage_token=faab02e1f62a58f7bad4c2ae5b8622e19846b97dde2a76f546c4bb1230cee044",
            headers=get_headers(base_url, "POST"),
            data=form_data,
        )
        if response.status_code == 200 and len(response.text) > 0:
            json_response = json.loads(response.text)
            form_data = {
                "_dummy": 1,
                "_session_storage": json.dumps(
                    {"_global": get_session_storage_global()}
                ),
                "_update_page_content_request": 1,
                "form_check_ajax": csrf_token,
            }
            response = s.post(
                base_url + json_response["redirect_url"],
                headers=get_headers(base_url, "POST"),
                data=form_data,
            )
            if response.status_code == 200 and len(response.text) > 0:
                json_response = json.loads(response.text)
                collection_data = json_response["data"]
            else:
                raise ValueError("Code 4: Failed to get bin data.")
        else:
            raise ValueError(
                "Code 5: Failed to get bin data. Too many requests. Please wait a few minutes before trying again."
            )
    return collection_data


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()
        s = requests.session()
        base_url = "https://service.croydon.gov.uk"
        paon = kwargs.get("paon")
        postcode = kwargs.get("postcode")
        check_paon(paon)
        check_postcode(postcode)

        # Firstly, get a CSRF (cross-site request forgery) token
        csrf_token = get_csrf_token(s, base_url)
        # Next, get the address_id
        address_id = get_address_id(s, base_url, csrf_token, postcode, paon)
        # Finally, use the address_id to get the collection data
        collection_data = get_collection_data(s, base_url, csrf_token, address_id)
        if collection_data != "":
            soup = BeautifulSoup(collection_data, features="html.parser")
            soup.prettify()

            # Find the list elements
            collection_record_elements = soup.find_all(
                "div", {"class": "listing_template_record"}
            )

            # Form a JSON wrapper
            data = {"bins": []}

            for e in collection_record_elements:
                collection_type = e.find_all(
                    "div", {"class": "fragment_presenter_template_show"}
                )[0].text.strip()
                collection_date = (
                    e.find("div", {"class": "bin-collection-next"})
                    .attrs["data-current_value"]
                    .strip()
                )
                dict_data = {
                    "type": collection_type,
                    "collectionDate": datetime.strptime(
                        collection_date, "%d/%m/%Y %H:%M"
                    ).strftime(date_format),
                }
                data["bins"].append(dict_data)

            if len(data["bins"]) == 0:
                raise ValueError(
                    "Code 5: No bin data found. Please ensure the council website is showing data first,"
                    " then open an issue on GitHub."
                )

            return data



import pytest
from pytest_bdd import scenario, given, when, then, parsers

from step_helpers import file_handler
import logging
import traceback

from uk_bin_collection.uk_bin_collection import collect_data


@scenario("../features/validate_council_outputs.feature", "Validate Council Output")
def test_scenario_outline():
    pass


@pytest.fixture
def context():
    class Context(object):
        pass

    return Context()


@given(parsers.parse("the council: {council_name}"))
def get_council_step(context, council_name):
    try:
        council_input_data = file_handler.load_inputs_file("input.json")
        context.metadata = council_input_data[council_name]
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Validate Output: {err}")
        raise (err)


@when(parsers.parse("we scrape the data from {council}"))
def scrape_step(context, council):
    context.council = council
    args = [council, context.metadata["url"]]

    if "uprn" in context.metadata:
        uprn = context.metadata["uprn"]
        args.append(f"-u={uprn}")
    if "postcode" in context.metadata:
        postcode = context.metadata["postcode"]
        args.append(f"-p={postcode}")
    if "house_number" in context.metadata:
        house_number = context.metadata["house_number"]
        args.append(f"-n={house_number}")
    if "usrn" in context.metadata:
        usrn = context.metadata["usrn"]
        args.append(f"-us={usrn}")
    if "SKIP_GET_URL" in context.metadata:
        args.append(f"-s")

    try:
        CollectData = collect_data.UKBinCollectionApp()
        CollectData.set_args(args)
        context.parse_result = CollectData.run()
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Schema: {err}")
        raise (err)


@then("the result is valid json")
def validate_json_step(context):
    try:
        valid_json = file_handler.validate_json(context.parse_result)
        assert valid_json is True
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Validate Output: {err}")
        raise (err)


@then("the output should validate against the schema")
def validate_output_step(context):
    try:
        council_schema = file_handler.load_schema_file(f"{context.council}.schema")
        schema_result = file_handler.validate_json_schema(
            context.parse_result, council_schema
        )
        assert schema_result is True
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Validate Output: {err}")
        raise (err)



import argparse
import importlib
import os
import sys

# We use this method to dynamically import the council processor
SRC_PATH = os.path.join("councils")
module_path = os.path.realpath(os.path.join(os.path.dirname(__file__), SRC_PATH))
sys.path.append(module_path)


class UKBinCollectionApp:
    def __init__(self):
        self.parser = argparse.ArgumentParser(description="")
        self.parser.add_argument("module", type=str, help="Name of council module to use")
        self.parser.add_argument(
            "URL", type=str, help="URL to parse - should be wrapped in double quotes"
        )
        self.parser.add_argument(
            "-p",
            "--postcode",
            type=str,
            help="Postcode to parse - should include a space and be wrapped in "
                 "double-quotes",
            required=False,
        )
        self.parser.add_argument(
            "-n", "--number", type=str, help="House number to parse", required=False
        )
        self.parser.add_argument(
            "-s",
            "--skip_get_url",
            action="store_true",
            help="Skips the generic get_url - uses one in council class",
            required=False,
        )
        self.parser.add_argument("-u", "--uprn", type=str, help="UPRN to parse", required=False)
        self.parser.add_argument(
            "-d",
            "--dev_mode",
            action="store_true",
            help="Enables development mode - creates/updates outputs .json file for the council on each run",
            required=False,
        )
        self.parsed_args = None

    def set_args(self, args):
        self.parsed_args = self.parser.parse_args(args)

    def get_council_module(self, council_module_str):
        return importlib.import_module(council_module_str)

    def client_code(self, get_bin_data_class, address_url, **kwargs) -> None:
        """
        The client code calls the template method to execute the algorithm. Client
        code does not have to know the concrete class of an object it works with,
        as long as it works with objects through the interface of their base class.
        """
        return get_bin_data_class.template_method(address_url, **kwargs)

    def run(self):
        council_module_str = self.parsed_args.module
        address_url = self.parsed_args.URL
        council_module = self.get_council_module(council_module_str)
        postcode = self.parsed_args.postcode
        paon = self.parsed_args.number
        uprn = self.parsed_args.uprn
        skip_get_url = self.parsed_args.skip_get_url
        dev_mode = self.parsed_args.dev_mode

        return self.client_code(
            council_module.CouncilClass(),
            address_url,
            postcode=postcode,
            paon=paon,
            uprn=uprn,
            skip_get_url=skip_get_url,
            dev_mode=dev_mode,
            council_module_str=council_module_str,
        )


if __name__ == "__main__":
    import sys

    app = UKBinCollectionApp()
    app.set_args(sys.argv[1:])
    data = app.run()
    print(data)



from datetime import *

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get UPRN and postcode from the parsed args
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        requests.packages.urllib3.disable_warnings()

        # Set up some form data, then POST for the form and scrape the result
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8",
            "Accept-Language": "en-GB,en;q=0.6",
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Content-Type": "multipart/form-data; boundary=----WebKitFormBoundaryI1XYcX9fNeKxm4LB",
            # 'Cookie': 'PHPSESSID=-3mn6j-vkWcY4xPPXbT3Ggk1gSQJLId%2CztSoQV5-f8Pi7Cju1wwE151qtwdUyE1c',
            "Origin": "https://www.tmbc.gov.uk",
            "Pragma": "no-cache",
            "Referer": "https://www.tmbc.gov.uk/xfp/form/167",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Sec-GPC": "1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "sec-ch-ua": '"Not?A_Brand";v="8", "Chromium";v="108", "Brave";v="108"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
        }
        data = (
            f"------WebKitFormBoundaryI1XYcX9fNeKxm4LB\r\nContent-Disposition: form-data; "
            f'name="__token"\r\n\r\ns_flSv1eIvJDeCwbFaYxclM3UTomdpWgg2cMWzZckaU\r\n'
            f"------WebKitFormBoundaryI1XYcX9fNeKxm4LB\r\nContent-Disposition: form-data; "
            f'name="page"\r\n\r\n128\r\n------WebKitFormBoundaryI1XYcX9fNeKxm4LB\r\nContent-Disposition: '
            f'form-data; name="locale"\r\n\r\nen_GB\r\n------WebKitFormBoundaryI1XYcX9fNeKxm4LB\r\nContent'
            f'-Disposition: form-data; name="q752eec300b2ffef2757e4536b77b07061842041a_0_0"\r\n\r\n'
            f"{user_postcode}\r\n------WebKitFormBoundaryI1XYcX9fNeKxm4LB\r\nContent-Disposition: form-data; "
            f'name="q752eec300b2ffef2757e4536b77b07061842041a_1_0"\r\n\r\n'
            f"{user_uprn}\r\n------WebKitFormBoundaryI1XYcX9fNeKxm4LB\r\nContent-Disposition: form-data; "
            f'name="next"\r\n\r\nNext\r\n------WebKitFormBoundaryI1XYcX9fNeKxm4LB--\r\n '
        )

        response = requests.post(
            "https://www.tmbc.gov.uk/xfp/form/167", headers=headers, data=data
        )
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}
        last_date = datetime.now()

        # Find the table on the page and get data from each row (we don't care about the headings)
        table = soup.find(
            "table", {"class": "data-table waste-collections-table"}
        ).find("tbody")
        for row in table.find_all("tr"):
            bin_date = row.find_next("td").text.strip()
            collection_types = row.find("div", {"class": "collections"}).find_all("p")

            # For each collection type in the list, parse the time
            for item in collection_types:
                curr_bin_date = datetime.strptime(bin_date, "%a %d %B")

                # The calendar doesn't include the year, so using this to try to mitigate year change (note: it's
                # currently January, so no idea if it will work until the end of the year lol)
                if last_date.date().isocalendar()[1] < 52:
                    curr_bin_date = datetime(
                        last_date.year, curr_bin_date.month, curr_bin_date.day
                    )
                else:
                    curr_bin_date = datetime(
                        last_date.year + 1, curr_bin_date.month, curr_bin_date.day
                    )

                # Add each collection to the dictionary
                dict_data = {
                    "type": item.text.strip(),
                    "collectionDate": curr_bin_date.strftime(date_format),
                }
                data["bins"].append(dict_data)
                last_date = curr_bin_date

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Create a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")

        bin_data_dict = {"bins": []}

        # Search for the specific table using BS4
        rows = soup.find("table", {"class": "job-details"}).find_all(
            "tr", {"class": "data-row"}
        )

        # Loop through the rows
        for row in rows:
            cells = row.find_all("td", {"class": lambda L: L and L.startswith("visible-cell")})

            bin_type = cells[0].find_all("label")[2].get_text(strip=True)
            collection_date = cells[0].find_all("label")[1].get_text(strip=True)

            # Create a dictionary for each Bin element in the JSON
            dict_data = {
                "type": bin_type,
                "collectionDate": collection_date,
            }

            # Add data to the main JSON Wrapper
            bin_data_dict["bins"].append(dict_data)

        return bin_data_dict



from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://selfservice.rushcliffe.gov.uk/renderform.aspx?t=1242&k=86BDCD8DE8D868B9E23D10842A7A4FE0F1023CCA"

        data = {"bins": []}

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Set up Selenium to run 'headless'
        options = webdriver.ChromeOptions()
        options.add_argument("--headless")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-gpu")
        options.add_argument("--disable-dev-shm-usage")
        options.add_experimental_option("excludeSwitches", ["enable-logging"])

        # Create Selenium webdriver
        driver = webdriver.Chrome(options=options)
        driver.get(page)

        # Populate postcode field
        inputElement_postcode = driver.find_element(
            By.ID,
            "ctl00_ContentPlaceHolder1_FF3518TB",
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        driver.find_element(
            By.ID,
            "ctl00_ContentPlaceHolder1_FF3518BTN",
        ).click()

        # Wait for the 'Select address' dropdown to appear and select option matching UPRN
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "ctl00_ContentPlaceHolder1_FF3518DDL"))
        )
        # Create a 'Select' for it, then select the matching URPN option
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value("U" + user_uprn)

        # Wait for the submit button to appear, then click it to get the collection dates
        submit = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "ctl00_ContentPlaceHolder1_btnSubmit"))
        )
        submit.click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        bins_text = soup.find("div", id="ctl00_ContentPlaceHolder1_pnlConfirmation")

        if bins_text:
            results = re.findall("Your (.*?) bin will next be collected on (\d\d?\/\d\d?\/\d{4})",
                                 bins_text.find("div", {"class": "ss_confPanel"}).get_text())
            if results:
                for result in results:
                    collection_date = datetime.strptime(result[1], "%d/%m/%Y")
                    dict_data = {
                        "type": result[0],
                        "collectionDate": collection_date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

                    data["bins"].sort(
                        key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
                    )

        return data



import time

import requests
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def extract_styles(self, style_str: str) -> dict:
        return dict(
            (a.strip(), b.strip())
            for a, b in (
                element.split(":") for element in style_str.split(";") if element
            )
        )

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://www.northumberland.gov.uk/Waste/Bins/Bin-Calendars.aspx"

        data = {"bins": []}

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_paon(user_paon)
        check_postcode(user_postcode)

        # Set up Selenium to run 'headless'
        options = webdriver.ChromeOptions()
        options.add_argument("--headless")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-gpu")
        options.add_argument("--disable-dev-shm-usage")
        options.add_experimental_option("excludeSwitches", ["enable-logging"])

        # Create Selenium webdriver
        driver = webdriver.Chrome(options=options)
        driver.get(page)

        time.sleep(1)

        # Press the cookie accept - wait is to let the JS load it up
        driver.find_element(By.ID, "ccc-notify-accept").click()

        inputElement_hn = driver.find_element(
            By.ID,
            "p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_NCCAddressLookup_txtHouse",
        )
        inputElement_pc = driver.find_element(
            By.ID,
            "p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_NCCAddressLookup_txtPostcode",
        )

        inputElement_pc.send_keys(user_postcode)
        inputElement_hn.send_keys(user_paon)

        driver.find_element(
            By.ID,
            "p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_NCCAddressLookup_butLookup",
        ).click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Work out which bins can be collected for this address. Glass bins are only on some houses due to pilot programme.
        bins_collected = list(
            map(
                str.strip,
                soup.find(
                    "span",
                    id="p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_spanRouteSummary",
                )
                .string.replace("Routes found: ", "")
                .split(","),
            )
        )

        # Get the background colour for each of them...
        bins_by_colours = dict()
        for bin in bins_collected:
            style_str = soup.find("span", string=bin)["style"]
            bin_colour = self.extract_styles(style_str)["background-color"].upper()
            bins_by_colours[bin_colour] = bin

        # Work through the tables gathering the dates, if the cell has a background colour - match it to the bin type.
        calander_tables = soup.find_all("table", title="Calendar")
        for table in calander_tables:
            # Get month and year
            # First row in table is the header
            rows = table.find_all("tr")
            month_and_year = (
                rows[0].find("table", class_="calCtrlTitle").find("td").string
            )
            bin_days = table.find_all("td", class_="calCtrlDay")
            for day in bin_days:
                day_styles = self.extract_styles(day["style"])
                if "background-color" in day_styles:
                    colour = day_styles["background-color"].upper()
                    date = time.strptime(f"{day.string} {month_and_year}", "%d %B %Y")

                    # Add it to the data
                    data["bins"].append(
                        {
                            "type": bins_by_colours[colour],
                            "collectionDate": time.strftime(date_format, date),
                        }
                    )

        return data



# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from dateutil.relativedelta import relativedelta
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        bin_data_dict = {"bins": []}
        collections = []


        # Search for the specific bins in the table using BS4
        bin_types = soup.find_all("h3", class_="govuk-heading-m waste-service-name")
        collection_info = soup.find_all("dl", {"class": "govuk-summary-list"})

        # Raise error if data is not loaded at time of scrape (30% chance it is)
        if len(bin_types) == 0:
            raise ConnectionError("Error fetching council data: data absent when page was scraped.")

        # Parse the data
        for idx, value in enumerate(collection_info):
            bin_type = bin_types[idx].text.strip()
            collection_date = value.contents[3].contents[3].text.strip()
            next_collection = datetime.strptime(remove_ordinal_indicator_from_date_string(collection_date.replace(',', '')), "%A %d %B")
            curr_date = datetime.now().date()
            next_collection = next_collection.replace(year=curr_date.year)
            if curr_date.month == 12 and next_collection.month == 1:
                next_collection = next_collection + relativedelta(years=1)
            collections.append((bin_type, next_collection))

        # Sort the text and list elements by date
        ordered_data = sorted(collections, key=lambda x: x[1])

        # Put the elements into the dictionary
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            bin_data_dict["bins"].append(dict_data)


        return bin_data_dict



import re
from datetime import datetime

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        url = "https://www.durham.gov.uk/bincollections?uprn="
        uprn = kwargs.get("uprn")
        check_uprn(uprn)
        url += uprn
        requests.packages.urllib3.disable_warnings()
        page = requests.get(url)

        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")

        data = {"bins": []}

        for bin_type in ["rubbish", "recycling", "gardenwaste"]:
            bin_info = soup.find(class_=f"bins{bin_type}")

            if bin_info:
                collection_text = bin_info.get_text(strip=True)

                if collection_text:
                    results = re.search("\\d\\d? [A-Za-z]+ \\d{4}", collection_text)
                    if results:
                        date = datetime.strptime(results[0], "%d %B %Y")
                        if date:
                            data["bins"].append({
                                "type": bin_type,
                                "collectionDate": date.strftime(date_format)
                            })

        return data
